@@ -16,7 +16,7 @@ def IsPython3():
 
 if not IsPython3():
     reload(sys)
-    sys.setdefaultencoding('utf-8') #WeiYF.20150114 强制转为 utf-8 编码
+    sys.setdefaultencoding('utf-8')  # WeiYF.20150114 强制转为 utf-8 编码
 
 
 def GetCurrentTime():
@@ -209,6 +209,77 @@ def ReturnWeekNumFmYMD(sYMD):
     return time.strftime('%U',tmStruct) # 以星期天为周的第一天
     # return  time.strftime('%W',tmStruct) # 以星期一为周的第一天
 
+def GetAttributeFromModule(sModuleName, sAttrName):
+    """使用importlib动态载入模块，考虑放入weberFuncs"""
+    import importlib
+    import traceback
+    try:
+        m = importlib.import_module(sModuleName)
+        return getattr(m, sAttrName)
+    except ImportError as e:
+        traceback.print_exc()  # WeiYF.20151022 打印异常整个堆栈 这个对于动态加载非常有用
+        PrintTimeMsg('GetAttributeFromModule(%s,%s).Exception={%s}!' % (
+            sModuleName, sAttrName, str(e)))
+        raise e  # 再次抛出该异常，不带e也一样
+
+
+def FormatDateTimeYmdHnsX(dtValue):
+    # 格式化datetime类型为 YmdHnsX 串
+    return dtValue.strftime('%Y%m%d-%H%M%S.%f')[:-3]
+
+
+def GetDateTimeFmYmdHnsX(sYmdHnsX):
+    # 将 YmdHnsX 串 转为datetime类型
+    from datetime import datetime
+    return datetime.strptime(sYmdHnsX, '%Y%m%d-%H%M%S.%f')
+
+
+def SubSecondsYmdHnsX(sYmdHnsX1, sYmdHnsX2):
+    # 将两个 YmdHnsX 格式的时间串相减得到的秒数
+    from datetime import datetime
+    dt1 = datetime.strptime(sYmdHnsX1, '%Y%m%d-%H%M%S.%f')
+    dt2 = datetime.strptime(sYmdHnsX2, '%Y%m%d-%H%M%S.%f')
+    fSeconds = (dt1 - dt2).total_seconds()
+    # PrintTimeMsg('SubSecondsYmdHnsX(%s,%s)=%ss=' % (sYmdHnsX1, sYmdHnsX2, fSeconds))
+    return fSeconds
+
+
+def AddSecondsYmdHnsX(sYmdHnsX, fSeconds):
+    # 为 YmdHnsX 格式的时间串加上指定的秒数
+    from datetime import datetime, timedelta
+    dt = datetime.strptime(sYmdHnsX, '%Y%m%d-%H%M%S.%f')
+    dt += timedelta(seconds=fSeconds)
+    sR = dt.strftime('%Y%m%d-%H%M%S.%f')[:len(sYmdHnsX)]
+    # PrintTimeMsg('AddSecondsYmdHnsX(%s,%f)=%s=' % (sYmdHnsX, fSeconds, sR))
+    return sR
+
+
+def SubSecondsHHNNSS(sE, sB):
+    # 将两个 HHNNSS 格式的时间串相减，不考虑日期因素
+    iHb = int(sB[0:2])
+    iNb = int(sB[2:4])
+    iSb = int(sB[4:6])
+    iHe = int(sE[0:2])
+    iNe = int(sE[2:4])
+    iSe = int(sE[4:6])
+    iSecsB = (iHb*60 + iNb) * 60 + iSb
+    iSecsE = (iHe*60 + iNe) * 60 + iSe
+    iSeconds = iSecsE - iSecsB
+    # PrintTimeMsg('SubSecondsHHNNSS(%s,%s)=%s=!' % (sE, sB, iSeconds))
+    return iSeconds
+
+
+def AddSeondsHHNNSS(sHNS, iSeconds):
+    # 为 HHNNSS 格式的时分秒串增加秒数，不考虑日期因素
+    iH = int(sHNS[0:2])
+    iN = int(sHNS[2:4])
+    iS = int(sHNS[4:6])
+    iTotalSeconds = (iH * 60 + iN) * 60+iS
+    iTotalSeconds += iSeconds
+    iMM, iSS = divmod(iTotalSeconds, 60)
+    iHH, iNN = divmod(iMM, 60)
+    iHH %= 24
+    return '%.2d%.2d%.2d' % (iHH, iNN, iSS)
 # ------------------------------
 
 
@@ -414,6 +485,26 @@ class ClassForAttachAttr(object):
         return 'ClassForAttachAttr=%s=' % ( str(self.__dict__))
 
 
+class CObjectOfDict(object):
+    """
+        WeiYF.20181030 为了更好设置保存动态属性，引入的类，支持传入dict
+    """
+    def __init__(self, dictInit={}):
+        if type(dictInit) == dict:
+            self.__dict__.update(dictInit)
+        else:
+            PrintTimeMsg("CObjectOfDict.type(dictInit) not dict!")
+
+    def __str__(self):
+        return 'CObjectOfDict=%s=' % (str(self.__dict__))
+
+    def get_dict(self):
+        return self.__dict__
+
+    def upd_dict(self, dictVal):
+        self.__dict__.update(dictVal)
+
+
 def GetSystemPlatform():
     # 返回当前操作系统类型
     import platform
@@ -499,8 +590,6 @@ def md5file(fname):
     return oHash.hexdigest()
 
 # -------------------------------
-
-
 def ReadTailLines(sFileName, total_lines_wanted):
     """
         从 sFileName 中读取尾部 total_lines_wanted 条记录
@@ -512,8 +601,7 @@ def ReadTailLines(sFileName, total_lines_wanted):
         block_end_byte = f.tell()
         lines_to_go = total_lines_wanted
         block_number = -1
-        blocks = [] # blocks of size BLOCK_SIZE, in reverse order starting
-                    # from the end of the file
+        blocks = []  # blocks of size BLOCK_SIZE, in reverse order starting from the end of the file
         while lines_to_go > 0 and block_end_byte > 0:
             if block_end_byte - BLOCK_SIZE > 0:
                 # read the last block we haven't yet read
@@ -521,14 +609,22 @@ def ReadTailLines(sFileName, total_lines_wanted):
                 blocks.append(f.read(BLOCK_SIZE))
             else:
                 # file too small, start from begining
-                f.seek(0,0)
+                f.seek(0, 0)
                 # only read what was not read
                 blocks.append(f.read(block_end_byte))
-            lines_found = blocks[-1].count('\n')
+            if IsPython3():
+                # PrintTimeMsg('DoHandleSendCmdToPeer.type(blocks)=%s=!' % type(blocks))
+                lines_found = blocks[-1].count(b'\n')
+            else:
+                lines_found = blocks[-1].count('\n')
             lines_to_go -= lines_found
             block_end_byte -= BLOCK_SIZE
             block_number -= 1
-        all_read_text = ''.join(reversed(blocks))
+        if IsPython3():
+            all_read_text = b''.join(reversed(blocks))
+            all_read_text = str(all_read_text, encoding='utf8')  # 转为串
+        else:
+            all_read_text = ''.join(reversed(blocks))
         return all_read_text.splitlines()[-total_lines_wanted:]
 
 
